#!/bin/bash

#==============================================================================
# epages functions
#==============================================================================

fun_epages_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export extract_host_from_url extract_port_from_url chkconfig_do

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export get_host_name

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

[[ -n $fun_java_sourced ]] || . $(dirname $BASH_SOURCE)/fun-java.sh
export mongod_name

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export epagesj_rpmsave_value write_epagesj_value
export start_epagesj update_mongod_conf update_epagesj_conf

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# start_epagesj
#..............................................................................
start_epagesj () {
  local USAGE='
syntax: start_epagesj
description: adds and starts epagesj/solr services
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local INIT_DIR="${RPM_INSTALL_PREFIX:-/srv/epages}"/etc/init.d

  chmod 755 "$INIT_DIR" "${INIT_DIR%/*}"
  for i in epages-solr epages-solr-slave epagesj ; do
    chmod 740 "$INIT_DIR"/$i
    chkconfig_do --del $i >/dev/null 2>&1
    [[ -f "$(readlink -qf /etc/init.d/$i)" ]] || ( cd /etc/init.d ; ln -fs "$INIT_DIR"/$i )
     chkconfig_do --add $i >/dev/null 2>&1
    chown root:epages "$INIT_DIR"/$i
    /etc/init.d/$i status >/dev/null && DO=restart || DO=start
    /etc/init.d/$i $DO
  done
}

#..............................................................................
# epagesj_rpmsave_value
#..............................................................................
epagesj_rpmsave_value () {
  local USAGE='
syntax: epagesj_rpmsave_value SECTION KEY
description: get value of KEY in SECTION of epagesj.conf.rpmsave
'
  [[ $# -ne 2 ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILE="$EPAGESJ_CONFIG"/epagesj.conf.rpmsave
  [[ -f $FILE ]] || { echo undef; return; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  $INI_CONFIG -f "$FILE" -s "$1" -G | grep -qP "^\s*$2\s*=" || {
    echo undef; return
  }
  $INI_CONFIG -f "$FILE" -s "$1" -k "$2" -G
}

#..............................................................................
# write_epagesj_value
#..............................................................................
write_epagesj_value () {
  local USAGE='
syntax: write_epagesj_value SECTION KEY NEW [ALT]
description: sets value of KEY in SECTION of epagesj.conf
  with the help of the OLD value in epagesj.conf.rpmsave
  a) if OLD is defined, use this value
  b) elsif ALT is defined and ALT value in *.rpmsave
     is defined, use this value
  c) else use NEW value
'
  [[ $# -lt 3 ]] && { echo "$USAGE" 1>&2; return 1; }

  local FILE="$EPAGESJ_CONFIG"/epagesj.conf
  local SECTION=$1
  local KEY=$2
  local NEW=$3
  local ALT=$4

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  OLD="$(epagesj_rpmsave_value "$1" "$2")" 
  if [[ $OLD = undef && -n $ALT ]] ; then
    local EXTRACT=$ALT
    ALT=${ALT#extract_host }
    ALT=${ALT#extract_port }
    OLD="$(epagesj_rpmsave_value "${ALT%%.*}" "${ALT#*.}")"
    [[ $OLD = undef && "${ALT%%.*}" = EPagesJServer ]] \
    && OLD="$(epagesj_rpmsave_value Server "${ALT#*.}")"
    if [[ $OLD != undef && $EXTRACT != ${EXTRACT#extract_host } ]] ; then
      OLD=$(extract_host_from_url "$OLD")
    elif [[ $OLD != undef && $EXTRACT != ${EXTRACT#extract_port } ]] ; then
      OLD=$(extract_port_from_url "$OLD")
      OLD=${OLD:-undef}
    fi
  fi

  [[ $OLD != undef ]] && NEW=$OLD
  $INI_CONFIG -f "$FILE" -s "$1" -k "$2" -v "\"$NEW\"" -i
}

#..............................................................................
# update_mongod_conf
#..............................................................................
update_mongod_conf () {
  local USAGE='
syntax: update_mongod_conf
description: updates /etc/mongod.conf as defined in /etc/sysconfig/epages6
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})

  [[ -f $(readlink -f "$ETC_DEFAULT") ]] && . "$ETC_DEFAULT"

  local CONF=${MONGOD_CONF:-/etc/$(mongod_name).conf}
  [[ -f $CONF ]] || return

  local used_port=$($INI_CONFIG -f "$CONF" -k port -G)
  if [[ ${EPAGES_INSTALL_MONGO_PORT:-27017} != ${used_port:-27017} ]] ; then
     $INI_CONFIG -f "$CONF" -k port -v "$EPAGES_INSTALL_MONGO_PORT" -i
  fi

  # if bind_ip is set and differs from MONGO_HOST, unset it
  local bind_ip=$($INI_CONFIG -f "$CONF" -k bind_ip -G)
  if [[ -n $bind_ip && $EPAGES_INSTALL_MONGO_HOST != $bind_ip ]] ; then
     sed -ri 's,^(\s*bind_ip\s*=.*),#\1,' "$CONF"
  fi

  smallfiles=$($INI_CONFIG -f "$CONF" -k smallfiles -G)
  [[ -z $smallfiles ]] && $INI_CONFIG -f "$CONF" -k smallfiles -v true -i
}

#..............................................................................
# update_epagesj_conf
#..............................................................................
update_epagesj_conf() {
  local USAGE='
syntax: update_epagesj_conf
description: equalizes variables between $EPAGESJ_CONFIG/epagesj.conf
  and /etc/sysconfig/epages6
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})
  local ETC_INSTALL=/etc/sysconfig/epages6

  [[ -f $(readlink -f "$ETC_DEFAULT") ]] && . "$ETC_DEFAULT"
  [[ -f $ETC_INSTALL ]] && . $ETC_INSTALL

  local CONF="${EPAGESJ_CONFIG:-/srv/epages/eproot/j/conf}"/epagesj.conf
  local SAMPLE=$(dirname "$CONF")/epagesj.sample.conf

  [[ -f "$SAMPLE" ]] || return
  rm -f "$CONF".rpmsave
  [[ -f "$CONF" ]] && mv "$CONF" "$CONF".rpmsave
  cp -p "$EPAGESJ_CONFIG"/epagesj.sample.conf "$CONF"

  # copy old values to new file

  if [[ -f "$CONF".rpmsave ]] ; then
    for section in $($INI_CONFIG -f "$CONF".rpmsave -G) ; do
      [[ $section = Server ]] && newsection=EPagesJServer || newsection=$section
      $INI_CONFIG -f "$CONF".rpmsave -s $section -G | sed "s,^,$newsection ,"
    done | while read newsection key is value ; do
      [[ -z $value ]] && continue
      $INI_CONFIG -f "$CONF" -s $newsection -k $key -v "$value" -i
    done
  fi

  # [Java]

  local EPAGESJ_OPTS=${EPAGESJ_SERVER_OPTS:-$EPAGES_INSTALL_EPAGESJ_OPTS}
  local JAVA_OPTS=$($INI_CONFIG -f "$SAMPLE" -s Java -k commandLineArguments -G)
  write_epagesj_value Java commandLineArguments "$JAVA_OPTS${EPAGESJ_OPTS:+ $EPAGESJ_OPTS}"

  local host port socket
  local defaultport=(8088 8983 8985 27017)
  local envvar=(EPAGESJ SOLR SOLRSLAVE MONGO)
  local defaulthost=$(get_host_name J)

  for i in 0 1 2 3 ; do
    host[$i]=$(eval echo '$'EPAGES_INSTALL_${envvar[$i]}_HOST)
    host[$i]=${host[$i]%%:ip6}
    host[$i]=${host[$i]:-$defaulthost}

    port[$i]=$(eval echo '$'EPAGES_INSTALL_${envvar[$i]}_PORT)
    port[$i]=$(repair_number "${port[$i]}")
    port[$i]=${port[$i]:-${defaultport[$i]}}

    if [[ ${host[$i]} = ${host[$i]#*:} ]] ; then
      socket[$i]="${host[$i]}:${port[$i]}"
    else # IPv6
      socket[$i]="[${host[$i]}]:${port[$i]}"
    fi
  done

  # [epagesj]

  write_epagesj_value epagesj server Server
  write_epagesj_value epagesj host "${host[0]}" EPagesJServer.host
  write_epagesj_value epagesj port "${port[0]}" EPagesJServer.port
  write_epagesj_value epagesj masterUrl "http://${socket[1]}/solr/" SolrClient.masterUrl
  write_epagesj_value epagesj slaveUrls "http://${socket[2]}/solr/" SolrClient.slaveUrls

  # [epages-solr]

  write_epagesj_value epages-solr server SolrServer
  write_epagesj_value epages-solr host "${host[1]}" "extract_host SolrClient.masterUrl"
  write_epagesj_value epages-solr port "${port[1]}" SolrServer.port

  # [epages-solr-slave]

  write_epagesj_value epages-solr-slave server SolrServer
  write_epagesj_value epages-solr-slave slave true
  write_epagesj_value epages-solr-slave host "${host[2]}" "extract_host SolrClient.slaveUrls"
  write_epagesj_value epages-solr-slave port "${port[2]}" "extract_port SolrClient.slaveUrls"
  write_epagesj_value epages-solr-slave masterUrl "http://${socket[1]}/solr/" SolrClient.masterUrl
  write_epagesj_value epages-solr-slave solrDirPrefix "slave_${port[2]}"

  # [Mongo]

  write_epagesj_value Mongo hosts "${socket[3]}"

  EPAGES_INSTALL_MONGO_HOST=${host[3]} EPAGES_INSTALL_MONGO_PORT=${port[3]} update_mongod_conf
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
