#!/bin/bash

#==============================================================================
# edit repositories
#==============================================================================

fun_repoedit_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL EPAGES_ARCH
export INI_CONFIG=/var/epages/ini-config.sh

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export get_arch_repo update_repo_arch
export update_repo_version update_repo_url
export update_yum_repo use_new_repos
export EPAGES_CAP3RD EPAGES_REPO3RD
export update_3rdparty_repos restore_3rdparty_repos
export delete_obsolete_repos enable_repo
export link_yum2zypper unlink_yum2zypper

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# get_arch_repo
#..............................................................................
get_arch_repo() {
  local USAGE='
syntax: get_arch_repo
returns: epages repo url which can be used as input for update_repo_url
example: update_repo_url $(get_arch_repo) setup
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local repo
  local repofile=/etc/yum.repos.d/epages-arch.repo
  [[ ! -f $repofile && -f $repofile.rpmsave ]] && repofile=$repofile.rpmsave
  if [[ -f $repofile ]] ; then
    repo=$(grep -P '^\s*baseurl' $repofile | head -1 |
           tr -d '[\042\047[:space:]]' | sed 's,[^=]*=,,;s,/kernel/.*,,')
  fi

  if [[ -z $repo ]] ; then
    echo "baseurl not found in: $repofile" 1>&2
    return 1
  else
    echo $repo
    return 0
  fi
}

#..............................................................................
# update_repo_arch
#..............................................................................
update_repo_arch() {
  local USAGE='
syntax: update_repo_arch REPO1 REPO2...
description: sets arch (x86_64/i386) in REPO1 REPO2...
example: update_repo_arch base arch setup
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local repo subst

  for repo in $@ ; do
    [[ -f /etc/yum.repos.d/epages-$repo.repo ]] || continue
    case $repo in
      base) subst="s,KERNEL,$EPAGES_KERNEL," ;;
      arch|setup) subst="s,ARCH,$EPAGES_ARCH,;s,KERNEL,$EPAGES_KERNEL," ;;
      *) continue ;; # other repos do not contain epages versions
    esac
    sed -ri "$subst" /etc/yum.repos.d/epages-$repo.repo
  done
}

#..............................................................................
# update_repo_version
#..............................................................................
update_repo_version() {
  local USAGE='
syntax: update_repo_version VERSION
description: updates epages apt repo to VERSION
example: update_repo_version 6.16.4
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] && return

  local subst='s,/apt/([6-9]\.[0-9]+-)?(updates|hotfix),/apt/'$1'-\2,;s,/apt/[6-9]\.[0-9]+\.[0-9]+,/apt/'$1','
  sed -ri "$subst" /etc/apt/sources.list.d/epages.list
}

#..............................................................................
# update_repo_url
#..............................................................................

update_repo_url() {
  local USAGE='
syntax: update_repo_url URL...
description: updates epages apt repo to URL
example: update_repo_url http://ep-server/yum/qa
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] && return

  local REPOURL=$1

  local subst='s, [^ ]+/apt/, '$REPOURL'/apt/,'
  sed -ri "$subst" /etc/apt/sources.list.d/epages.list

  # REPOHOST in http://REPOHOST/foobar
  local REPOHOST=${REPOURL#*://}
  # return if REPOURL has no ://
  [[ $REPOHOST = $REPOURL ]] && return

  local EPAGES_PREF=/etc/apt/preferences.d/epages-repo.pref
  mkdir -m 755 -p ${EPAGES_PREF%/*}
  /bin/echo -e "\nPackage: *\nPin: origin ${REPOHOST%%/*}\nPin-Priority: 703" > $EPAGES_PREF
  chmod 644 $EPAGES_PREF
}

#..............................................................................
# use_new_repos
#..............................................................................

use_new_repos() {
  local USAGE='
syntax: use_new_repos
description: moves .repo.rpmnew to .repo
example: use_new_repos
'
  [[ $# -gt 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  pushd /etc/yum.repos.d >/dev/null 2>&1 || return

  for i in $(ls -d *.repo 2>/dev/null) ; do
    grep -q templateurl $i && continue
    grep -q templateurl $i.rpmnew 2>/dev/null || continue
    /bin/mv $i.rpmnew $i
  done
  for i in $(ls -d mirrors-epages-*.rpmnew 2>/dev/null) ; do
    /bin/mv $i ${i%.rpmnew}
  done

  popd >/dev/null
}

#..............................................................................
# update_yum_repo
#..............................................................................

update_yum_repo() {
  local USAGE='
syntax: update_yum_repo REPO1 REPO2...
description: updates REPO1 REPO2... as defd in /etc/yum/vars/EPAGES_*
example: update_yum_repo setup update empty
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local EPAGES_VERSION=yum
  local EPAGES_REPO=ftp://epages-software.de/repo/epages
  local CUSTOMIZED
  if [[ -s /etc/yum/vars/EPAGES_VERSION ]] ; then
    EPAGES_VERSION=$(cat /etc/yum/vars/EPAGES_VERSION)
    CUSTOMIZED=1
  fi
  if [[ -s /etc/yum/vars/EPAGES_REPO ]] ; then
    EPAGES_REPO=$(cat /etc/yum/vars/EPAGES_REPO)
    CUSTOMIZED=1
  fi

  [[ -z $CUSTOMIZED ]] && return

  local file url
  for i in $@ ; do
    file=/etc/yum.repos.d/epages-$i.repo
    [[ -f $file ]] || continue
    for section in $($INI_CONFIG -f $file -G) ; do
      # read templateurl
      url=$($INI_CONFIG -f $file -s $section -k templateurl -G)
      [[ -z $url ]] && continue
      # set EPAGES_REPO and EPAGES_VERSION
      url=$(eval echo $url)
      # write baseurl
      $INI_CONFIG -f $file -s $section -k baseurl -v $url -i
    done
    # if customized don't use mirrorlist
    sed -ri 's,^([^#].*irrorlist.*),#\1,' $file
  done
}

#..............................................................................
# enable_repo
#..............................................................................
enable_repo() {
  local USAGE='
syntax: enable_repo [-e] {0|1} REPO1 REPO2...
description: enable (1) or disable (0) REPO1 REPO2... (with perl regexp)
options:
     -e: prepend epages- to each REPO (shorthand for epages- repos)
returns: changed repos
example: enable_repo -e 0 empty update >/dev/null
'
  [[ $# -lt 2 ]] && { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  local file section value arg prefix
  [[ $1 = -e ]] && { prefix=epages-; shift; }
  local ENABLE=$1 ; shift
  [[ $ENABLE = 1 ]] && arg=-e || arg=-d
  local PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin

  pushd /etc/yum.repos.d >/dev/null 2>&1 || return
  local CHANGED=$(echo "$@" | tr ' ' "\n" | while read i ; do
    grep -P '\['$prefix$i'\]' $prefix*.repo 2>/dev/null | while IFS=: read file section ; do 
      [[ -z $section ]] && continue
      section=${section//[\[\]]/}
      value=$($INI_CONFIG -f $file -s $section -k enabled -G)
      [[ -z $value ]] && continue
      [[ $value = $ENABLE ]] && continue

      [[ ${EPAGES_KERNEL%s} = $EPAGES_KERNEL ]] || zypper mr $arg $section

      $INI_CONFIG -f $file -s $section -k enabled -v $ENABLE -i
      echo -n "${section#$prefix} "
    done
  done)
  [[ -n $CHANGED ]] && echo $CHANGED
  popd >/dev/null
}

#..............................................................................
# delete_obsolete_repos
#..............................................................................
delete_obsolete_repos() {
  local USAGE='
syntax: delete_obsolete_repos
description: deletes epages repos not necessary anymore
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  pushd /etc/yum.repos.d >/dev/null 2>&1 || return
  rm -f mirrors-epages{,-kernel-*,-arch-*-*}
  rm -f epages{,-dev,-archive,-update-archive,-kernel}.repo{,.rpm*}
  popd >/dev/null
}

#..............................................................................
# EPAGES_CAP3RD EPAGES_REPO3RD
#..............................................................................
# third party capabilities
EPAGES_CAP3RD=(graphviz nodejs mp3splt mongo-10gen-server mod_rpaf)
# are provided by third party repositories
case $EPAGES_KERNEL in
  *s) EPAGES_REPO3RD=("" nodejs-sles packman-multimedia 10gen "") ;;
  *r) EPAGES_REPO3RD=(graphviz-rhel "" atrpms 10gen CentALT) ;;
  *d) EPAGES_REPO3RD=() ;;
esac

#..............................................................................
# update_3rdparty_repos
#..............................................................................
update_3rdparty_repos() {
  local USAGE='
syntax: update_3rdparty_repos
description: updates 3rd-party repositories in linux distribution
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%d} ]] && return

  local file
  for i in `seq 0 $((${#EPAGES_CAP3RD[@]} -1))` ; do
    [[ -z ${EPAGES_REPO3RD[$i]} ]] && continue
    file=/etc/yum.repos.d/${EPAGES_REPO3RD[$i]}.repo
    [[ -f $file ]] || continue
    cp -p $file $file.rpmsave
    case ${EPAGES_REPO3RD[$i]}-$EPAGES_KERNEL in
      10gen*) [[ $EPAGES_ARCH != x86_64 ]] && sed -i 's,\$basearch,i686,g' $file ;;
      nodejs-sles-2.6.27s) sed -i 's,_SP1,,g' $file ;;
      CentALT-2.6.18r) sed -i 's,\$releasever,5,g' $file ;;
      CentALT-2.6.32r) sed -i 's,\$releasever,6,g' $file ;;
    esac
  done
}

#..............................................................................
# restore_3rdparty_repos
#..............................................................................
restore_3rdparty_repos() {
  local USAGE='
syntax: restore_3rdparty_repos
description: restores 3rd-party repositories in linux distribution
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return

  for i in ${EPAGES_REPO3RD[@]} ; do
     [ -f /etc/yum.repos.d/$i.rpmsave ] && mv /etc/yum.repos.d/$i.rpmsave /etc/yum.repos.d/$i
  done
}

#..............................................................................
# link_yum2zypper
#..............................................................................
link_yum2zypper() {
  local USAGE='
syntax: link_yum2zypper
description: add linked yum repos to zypper
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  pushd /etc/zypp/repos.d >/dev/null 2>&1 || return
  for i in ${EPAGES_REPO3RD[@]} /etc/yum.repos.d/epages*.repo ; do
    [[ -e "/etc/yum.repos.d/$i.repo" ]] && i="/etc/yum.repos.d/$i.repo"
    [[ ! -f "$i" || $(find "${i##*/}" -printf "%l" 2>/dev/null) = "$i" ]] && continue
    rm -f ${i##*/} 2>/dev/null
    ln -s $i .
  done
  popd >/dev/null
}

#..............................................................................
# unlink_yum2zypper
#..............................................................................
unlink_yum2zypper() {
  local USAGE='
syntax: unlink_yum2zypper
description: remove linked yum repos from zypper
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  pushd /etc/zypp/repos.d >/dev/null 2>&1 || return
  for i in $(find * -type l 2>/dev/null) ; do
    l=$(find "$i" -printf "%l")
    [[ ${l#/etc/yum.repos.d/} = $l ]] && continue
    rm -f $i
  done
  popd >/dev/null
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
